@@ -18,6 +18,8 @@
  *   Boston, MA 02111-1307, USA.
  */
 
+#include <config.h>
+
 #include "glib.h"
 
 #include <string.h>
@@ -28,7 +30,7 @@
 #include "glibintl.h"
 
 GQuark
-g_markup_error_quark ()
+g_markup_error_quark (void)
 {
   static GQuark error_quark = 0;
 
@@ -48,7 +50,8 @@ typedef enum
   STATE_INSIDE_ATTRIBUTE_NAME,
   STATE_BETWEEN_ATTRIBUTES,
   STATE_AFTER_ATTRIBUTE_EQUALS_SIGN,
-  STATE_INSIDE_ATTRIBUTE_VALUE,
+  STATE_INSIDE_ATTRIBUTE_VALUE_SQ,
+  STATE_INSIDE_ATTRIBUTE_VALUE_DQ,
   STATE_INSIDE_TEXT,
   STATE_AFTER_CLOSE_TAG_SLASH,
   STATE_INSIDE_CLOSE_TAG_NAME,
@@ -94,6 +97,7 @@ struct _GMarkupParseContext
 
   guint document_empty : 1;
   guint parsing : 1;
+  gint balance;
 };
 
 /**
@@ -151,6 +155,8 @@ g_markup_parse_context_new (const GMarkupParser *parser,
   context->document_empty = TRUE;
   context->parsing = FALSE;
 
+  context->balance = 0;
+
   return context;
 }
 
@@ -267,6 +273,44 @@ utf8_str (const gchar *utf8,
   return buf;
 }
 
+static gboolean
+str_has_suffix (const gchar  *str,
+		const gchar  *suffix)
+{
+  int str_len;
+  int suffix_len;
+  
+  g_return_val_if_fail (str != NULL, FALSE);
+  g_return_val_if_fail (suffix != NULL, FALSE);
+
+  str_len = strlen (str);
+  suffix_len = strlen (suffix);
+
+  if (str_len < suffix_len)
+    return FALSE;
+
+  return strcmp (str + str_len - suffix_len, suffix) == 0;
+}
+
+static gboolean
+str_has_prefix (const gchar  *str,
+		const gchar  *prefix)
+{
+  int str_len;
+  int prefix_len;
+  
+  g_return_val_if_fail (str != NULL, FALSE);
+  g_return_val_if_fail (prefix != NULL, FALSE);
+
+  str_len = strlen (str);
+  prefix_len = strlen (prefix);
+
+  if (str_len < prefix_len)
+    return FALSE;
+  
+  return strncmp (str, prefix, prefix_len) == 0;
+}
+
 static void
 set_unescape_error (GMarkupParseContext *context,
                     GError             **error,
@@ -589,9 +633,27 @@ unescape_text (GMarkupParseContext *context,
         }
     }
 
-  /* If no errors, we should have returned to USTATE_INSIDE_TEXT */
-  g_assert (context->state == STATE_ERROR ||
-            state == USTATE_INSIDE_TEXT);
+  if (context->state != STATE_ERROR) 
+    {
+      switch (state) 
+	{
+	case USTATE_INSIDE_TEXT:
+	  break;
+	case USTATE_AFTER_AMPERSAND:
+	case USTATE_INSIDE_ENTITY_NAME:
+	  set_unescape_error (context, error,
+			      NULL, NULL,
+			      G_MARKUP_ERROR_PARSE,
+			      _("Unfinished entity reference"));
+	  break;
+	case USTATE_AFTER_CHARREF_HASH:
+	  set_unescape_error (context, error,
+			      NULL, NULL,
+			      G_MARKUP_ERROR_PARSE,
+			      _("Unfinished character reference"));
+	  break;
+	}
+    }
 
   if (context->state == STATE_ERROR)
     {
@@ -623,12 +685,18 @@ advance_char (GMarkupParseContext *context)
   return context->iter != context->current_text_end;
 }
 
+static gboolean
+xml_isspace (char c)
+{
+  return c == ' ' || c == '\t' || c == '\n' || c == '\r';
+}
+
 static void
 skip_spaces (GMarkupParseContext *context)
 {
   do
     {
-      if (!g_unichar_isspace (g_utf8_get_char (context->iter)))
+      if (!xml_isspace (*context->iter))
         return;
     }
   while (advance_char (context));
@@ -695,7 +763,7 @@ find_current_text_end (GMarkupParseContext *context)
   p = context->current_text;
   next = g_utf8_find_next_char (p, end);
 
-  while (next)
+  while (next && *next)
     {
       p = next;
       next = g_utf8_find_next_char (p, end);
@@ -936,6 +1004,7 @@ g_markup_parse_context_parse (GMarkupParseContext *context,
               const gchar *openangle = "<";
               add_to_partial (context, openangle, openangle + 1);
               context->start = context->iter;
+	      context->balance = 1;
               context->state = STATE_INSIDE_PASSTHROUGH;
             }
           else if (*context->iter == '/')
@@ -1213,11 +1282,17 @@ g_markup_parse_context_parse (GMarkupParseContext *context,
           break;
 
         case STATE_AFTER_ATTRIBUTE_EQUALS_SIGN:
-          /* Possible next state: INSIDE_ATTRIBUTE_VALUE */
+          /* Possible next state: INSIDE_ATTRIBUTE_VALUE_[SQ/DQ] */
           if (*context->iter == '"')
             {
               advance_char (context);
-              context->state = STATE_INSIDE_ATTRIBUTE_VALUE;
+              context->state = STATE_INSIDE_ATTRIBUTE_VALUE_DQ;
+              context->start = context->iter;
+            }
+          else if (*context->iter == '\'')
+            {
+              advance_char (context);
+              context->state = STATE_INSIDE_ATTRIBUTE_VALUE_SQ;
               context->start = context->iter;
             }
           else
@@ -1235,15 +1310,28 @@ g_markup_parse_context_parse (GMarkupParseContext *context,
             }
           break;
 
-        case STATE_INSIDE_ATTRIBUTE_VALUE:
+        case STATE_INSIDE_ATTRIBUTE_VALUE_SQ:
+        case STATE_INSIDE_ATTRIBUTE_VALUE_DQ:
           /* Possible next states: BETWEEN_ATTRIBUTES */
-          do
-            {
-              if (*context->iter == '"')
-                break;
-            }
-          while (advance_char (context));
+	  {
+	    gchar delim;
 
+	    if (context->state == STATE_INSIDE_ATTRIBUTE_VALUE_SQ) 
+	      {
+		delim = '\'';
+	      }
+	    else 
+	      {
+		delim = '"';
+	      }
+
+	    do
+	      {
+		if (*context->iter == delim)
+		  break;
+	      }
+	    while (advance_char (context));
+	  }
           if (context->iter == context->current_text_end)
             {
               /* The value hasn't necessarily ended. Merge with
@@ -1385,7 +1473,19 @@ g_markup_parse_context_parse (GMarkupParseContext *context,
               close_name = g_string_free (context->partial_chunk, FALSE);
               context->partial_chunk = NULL;
               
-              if (context->tag_stack == NULL)
+              if (*context->iter != '>')
+                {
+                  gchar buf[7];
+                  set_error (context,
+                             error,
+                             G_MARKUP_ERROR_PARSE,
+                             _("'%s' is not a valid character following "
+                               "the close element name '%s'; the allowed "
+                               "character is '>'"),
+                             utf8_str (context->iter, buf),
+                             close_name);
+                }
+              else if (context->tag_stack == NULL)
                 {
                   set_error (context,
                              error,
@@ -1404,18 +1504,6 @@ g_markup_parse_context_parse (GMarkupParseContext *context,
                              close_name,
                              current_element (context));
                 }
-              else if (*context->iter != '>')
-                {
-                  gchar buf[7];
-                  set_error (context,
-                             error,
-                             G_MARKUP_ERROR_PARSE,
-                             _("'%s' is not a valid character following "
-                               "the close element name '%s'; the allowed "
-                               "character is '>'"),
-                             utf8_str (context->iter, buf),
-                             close_name);
-                }
               else
                 {
                   GError *tmp_error;
@@ -1447,13 +1535,28 @@ g_markup_parse_context_parse (GMarkupParseContext *context,
               g_free (close_name);
             }
           break;
-
+	  
         case STATE_INSIDE_PASSTHROUGH:
           /* Possible next state: AFTER_CLOSE_ANGLE */
           do
             {
-              if (*context->iter == '>')
-                break;
+	      if (*context->iter == '<') 
+		context->balance++;
+              if (*context->iter == '>') 
+		{
+		  context->balance--;
+		  add_to_partial (context, context->start, context->iter);
+		  context->start = context->iter;
+		  if ((str_has_prefix (context->partial_chunk->str, "<?")
+		       && str_has_suffix (context->partial_chunk->str, "?")) ||
+		      (str_has_prefix (context->partial_chunk->str, "<!--")
+		       && str_has_suffix (context->partial_chunk->str, "--")) ||
+		      (str_has_prefix (context->partial_chunk->str, "<![CDATA[") 
+		       && str_has_suffix (context->partial_chunk->str, "]]")) ||
+		      (str_has_prefix (context->partial_chunk->str, "<!DOCTYPE")
+		       && context->balance == 0)) 
+		    break;
+		}
             }
           while (advance_char (context));
 
@@ -1476,13 +1579,25 @@ g_markup_parse_context_parse (GMarkupParseContext *context,
               advance_char (context); /* advance past close angle */
               add_to_partial (context, context->start, context->iter);
 
-              if (context->parser->passthrough)
-                (*context->parser->passthrough) (context,
-                                                 context->partial_chunk->str,
-                                                 context->partial_chunk->len,
-                                                 context->user_data,
-                                                 &tmp_error);
-                  
+	      if (str_has_prefix (context->partial_chunk->str, "<![CDATA[")) 
+		{
+		  if (context->parser->text)
+		    (*context->parser->text) (context,
+					      context->partial_chunk->str + 9,
+					      context->partial_chunk->len - 12,
+					      context->user_data,
+					      &tmp_error);
+		}
+	      else 
+		{
+		  if (context->parser->passthrough)
+		    (*context->parser->passthrough) (context,
+						     context->partial_chunk->str,
+						     context->partial_chunk->len,
+						     context->user_data,
+						     &tmp_error);
+		}
+
               truncate_partial (context);
 
               if (tmp_error == NULL)
@@ -1599,7 +1714,8 @@ g_markup_parse_context_end_parse (GMarkupParseContext *context,
                    "following an attribute name; no attribute value"));
       break;
 
-    case STATE_INSIDE_ATTRIBUTE_VALUE:
+    case STATE_INSIDE_ATTRIBUTE_VALUE_SQ:
+    case STATE_INSIDE_ATTRIBUTE_VALUE_DQ:
       set_error (context, error, G_MARKUP_ERROR_PARSE,
                  _("Document ended unexpectedly while inside an attribute "
                    "value"));
