@@ -20,6 +20,8 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include <config.h>
+
 #include "glib.h"
 #include <string.h>
 
@@ -303,6 +305,7 @@ g_shell_unquote (const gchar *quoted_string,
         }
     }
 
+  g_free (unquoted);
   return g_string_free (retval, FALSE);
   
  error:
@@ -523,7 +526,7 @@ tokenize_command_line (const gchar *command_line,
         g_set_error (error,
                      G_SHELL_ERROR,
                      G_SHELL_ERROR_BAD_QUOTING,
-                     _("Text ended just after a '\' character."
+                     _("Text ended just after a '\\' character."
                        " (The text was '%s')"),
                      command_line);
       else
