@@ -37,6 +37,8 @@
 
 #include <glib.h>
 
+#ifdef G_THREADS_ENABLED
+
 static gboolean thread_system_already_initialized = FALSE;
 static gint g_thread_priority_map [G_THREAD_PRIORITY_URGENT + 1];
 
@@ -63,6 +65,7 @@ static gint g_thread_priority_map [G_THREAD_PRIORITY_URGENT + 1];
 void g_mutex_init (void);
 void g_mem_init (void);
 void g_messages_init (void);
+void g_convert_init (void);
 
 #define G_MUTEX_DEBUG_INFO(mutex) (*((gpointer*)(((char*)mutex)+G_MUTEX_SIZE)))
 
@@ -285,7 +288,7 @@ g_thread_init_with_errorcheck_mutexes (GThreadFunctions* init)
    * of g_thread_functions_for_glib_use_default based on operating
    * system version, C library version, or whatever. */
   g_thread_impl_init();
-#endif
+#endif /* HAVE_G_THREAD_IMPL_INIT */
 
   errorcheck_functions = g_thread_functions_for_glib_use_default;
   errorcheck_functions.mutex_new = g_mutex_new_errorcheck_impl;
@@ -311,10 +314,6 @@ g_thread_init (GThreadFunctions* init)
 {
   gboolean supported;
 
-#ifndef	G_THREADS_ENABLED
-  g_error ("GLib thread support is disabled.");
-#endif	/* !G_THREADS_ENABLED */
-
   if (thread_system_already_initialized)
     g_error ("GThread system may only be initialized once.");
     
@@ -351,7 +350,7 @@ g_thread_init (GThreadFunctions* init)
 	       init->cond_free &&
 	       init->private_new &&
 	       init->private_get &&
-	       init->private_get &&
+	       init->private_set &&
 	       init->thread_create &&
 	       init->thread_yield &&
 	       init->thread_join &&
@@ -381,6 +380,7 @@ g_thread_init (GThreadFunctions* init)
   g_mutex_init ();
   g_mem_init ();
   g_messages_init ();
+  g_convert_init ();
 
   /* now we can set g_threads_got_initialized and thus enable
    * all the thread functions
@@ -390,3 +390,13 @@ g_thread_init (GThreadFunctions* init)
   /* we want the main thread to run with normal priority */
   g_thread_set_priority (g_thread_self(), G_THREAD_PRIORITY_NORMAL);
 }
+
+#else /* !G_THREADS_ENABLED */
+
+void
+g_thread_init (GThreadFunctions* init)
+{
+  g_error ("GLib thread support is disabled.");
+}
+
+#endif /* !G_THREADS_ENABLED */
